/*
 * CDInfo.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


package base;

import java.util.*;



/**
 * Class for CD data; it is used to store information for a full CD (= "album"); the stored
 * information are used for displaying the CD within the list of albums of an artist, to
 * synchronize with the database and others. Internally a CDInfo object holds a list of
 * TrackInfo objects that belong to the tracks of this CD
 */
public class CDInfo
{
   public int               idx,year,tracks,wasCDDB=-1;
   public String            artist=null,album=null,genre=null,cat=null,discid=null,tracksid=null;
   public Vector<TrackInfo> trackList=null;


   
  /**
   * The default constructor
   */
   public CDInfo()
   {
   }
   
   

  /**
   * The standard constructor for CDInfo objects
   * @param idx an unique identifier for this element
   * @param Artist the name of the artist of this CD
   * @param Album the name of the Album/of this CD
   * @param Genre the genre of this album
   * @param Cat an optinally (currently not displayed) category information stored in parallel
   *        to the genre
   * @param Year the year this CD was published
   * @param Tracks the number of songs/titles/tracks on this CD
   */   
   public CDInfo(int idx,String Artist,String Album, String Genre, String Cat,int Year,int Tracks)
   {
      this.idx=idx;
      this.artist=Artist;
      this.album=Album;
      this.genre=Genre;
      this.cat=Cat;
      this.year=Year;
      this.tracks=Tracks;
   }
   
   
   
  /**
   * This methond compares two albums with each other
   * @param compare the CDInfo object to compare with
   * @return true in case the unique identifier idx is the same or in case the name of the artis
   *         and of the album are identically; false otherwise
   */    
   public boolean equals(CDInfo compare)
   {
      if ((idx>0) && (idx==compare.idx)) return true;
      if ((artist.equalsIgnoreCase(compare.artist)) &&
          (album.equalsIgnoreCase(compare.album))) return true;
      return false;
   }
   
   
   
  /**
   * Adds a new track to the album and appends it to the list of tracks; resulting from that all
   * tracks have to be appended ordered, there is no sorting mechanism for them
   * @param trackInfo a object of type TrackInfo that has to be added to the internal track list
   */
   public void addTrack(TrackInfo trackInfo)
   {
      if (trackList==null) trackList=new Vector<TrackInfo>();
      trackList.add(trackInfo);
   }



  /**
   * This method returns information about a track at a specific position
   * @param pos the number of the track the information are requested for
   * @return the TrackInfo object that contains the information for the track at the given position
   *         or null if no such track exists
   */
   public TrackInfo getTrackAt(int pos)
   {
      return trackList.elementAt(pos);
   }
   
   
  /**
   * Evaluates the number of tracks that are managed by this object; the number returned here is equal to
   * the number of TrackInfos that are really available for this CDInfo object
   * @return the number of available TrackInfos or -1 if no TrackInfo is stored; 0is never returned and
   *         reserved for future use 
   */
   public int getTracksNumber()
   {
      if (trackList==null) return -1;
      return trackList.size();
   }

   
  /**
   * Set a DiscID for this CD
   * @param id the DiscID like it is used by CDDB; here an extended DiscID is used that not only consists
   *        of the checksum of the CD but also of a list of time information for every track because only
   *        that ID is unique; please refer to the description of the "discid" utility for the format and
   *        meaning of this extended ID
   */
   public void setID(String id)
   {
      int pos=id.indexOf(" ");
      discid=id.substring(0,pos);
      tracksid=id.substring(pos+1);
   }
   
   
  /**
   * Sets new CDInfo parameters for the own object
   * @param info the new CDInfo data that have to be set; here only these fields are used that contain
   *        valid data, if members of info are uninitialized the related members of the target object are
   *        left unchanged
   */
   public void set(CDInfo info)
   {
      if (info==null) return;
      if (info.idx!=0) this.idx=info.idx; 
      if (info.year!=0) this.year=info.year; 
      if (info.tracks!=0) this.tracks=info.year; 
      if (info.artist!=null) this.artist=info.artist; 
      if (info.album!=null) this.album=info.album; 
      if (info.genre!=null) this.genre=info.genre; 
      if (info.cat!=null) this.cat=info.cat;
      if (info.discid!=null) this.discid=info.discid;
      if (info.tracksid!=null) this.tracksid=info.tracksid;
      if (info.getTracksNumber()>0)
      {
         if (trackList!=null) trackList.removeAllElements();
         for (int i=0; i<info.getTracksNumber(); i++)
          addTrack(info.getTrackAt(i));
      }
      if (info.wasCDDB!=-1) wasCDDB=info.wasCDDB;
   }
   
   
   
  /**
   * The position and path of an album within the directory that is configured for storing CD data can be
   * evaluated out of the CDInfo data by using this function
   * @return the full pathname to the directory where the data of this CD are stored into
   */
   public String getAlbumDir()
   {
      return PoddyFrame.MyWin.sysAccess.getMainDirectory()+"cds/"+artist+"/"+album;
   }
   
   
}
