/*
 * CustomPanel.java
 *
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */



package base;

import javax.swing.*;

import java.awt.*;



/**
 * This is the base class for all panels that are displayed from time to time to
 * offer additional user interaction. These custom panels are added to the main
 * window by calling PoddyFrame.setCustomPanel() and replace the standard media info listing.
 * This class inherits from JScrollComponent to offer vertical scolling functionality for
 * istings that are bigger than the available space on the screen.
 */
@SuppressWarnings("serial")
public class CustomPanel extends JScrollPane 
{

   public  static final int MAX_IMAGES=18;
   public  JFixedPanel      mainPanel=new JFixedPanel();
   public  int              selected=-1,prevSelected=-1;



   /**
    * Default constructor for a CustomPanel. It sets up a panel and adds a scroll pane
    * to it so that the contents within the panel can be bigger than the available display.
    * All subsequent calls to add() are catched within this class to act on the scroll pane
    * and not on the CustomPanel itself. Because of that structure you mustn't set a layout
    * manager for the scroll pane, it automatically uses GridBagLayout. 
    */
   public CustomPanel()
   {
	  setBorder(null);
      setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
      setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
      setBackground(PoddyFrame.bgcolor);
      getViewport().setOpaque(false);
      getViewport().add(mainPanel);
      getViewport().setScrollMode(JViewport.BLIT_SCROLL_MODE);
      mainPanel.setLayout(new GridBagLayout());
   }



  /**
   * Whenever a item within the list of elements (e.g. Artists or DVD titles) is selected it
   * should be visible to the user. That is important especially when a list is bigger than the
   * visible area of the main window. This method can be called to scroll within the current
   * CustomPanel so that the selected component becomes visible.
   * @param component the component that has to be scrolled to the visible area
   * @param asyncScroll when this value is set to true scrolling is not done immediately but
   *        later when there is time for it
   */
   public void scrollComponentToVisible(JComponent component,boolean asyncScroll)
   {
      if (asyncScroll) PoddyFrame.MyWin.scrollToObjectASync(component);
      else this.scrollComponentToVisible(component);
   }
   
   
  
  /**
   * This method overwrites the related paintComponent() method from the base class, here the
   * background of the panel is painted (together with a background image that may be configured for
   * the application.<BR>
   * This method never has to be called directly but only via repaint-actions!
   */
   public void paintComponent(Graphics g)
   {
      if (PoddyFrame.MyWin.bgImage!=null)
      {
         if ((PoddyFrame.MyWin.useImage==null) || 
             (PoddyFrame.MyWin.useImage.getWidth()<getWidth()) ||
        	 (PoddyFrame.MyWin.useImage.getHeight()<getHeight()))
          PoddyFrame.MyWin.useImage=PoddyFrame.MyWin.iu.scaleBufferedImage(PoddyFrame.MyWin.bgImage,getWidth(),getHeight());
         g.drawImage(PoddyFrame.MyWin.useImage,0,0,getWidth(),getHeight(),this);         
      }
      else super.paintComponent(g);
   }   

   
  
  /**
   * Whenever a item within the list of elements (e.g. Artists or DVD titles) is selected it
   * should be visible to the user. That is important especially when a list is bigger than the
   * visible area of the main window. This method can be called to scroll within the current
   * CustomPanel immediately so that the selected component becomes visible. Here a syncronous
   * scrolling operation is executes so that a call to this method is the same like calling
   * scrollComponentToVisible(component,false);
   * @param component the component that has to be scrolled to the visible area
   */
   public void scrollComponentToVisible(JComponent component)
   {
      if (getViewport().getSize().height<getViewport().getView().getHeight())
      {
         Rectangle rect;
         int y=0;

         rect=component.getBounds();      

         y=rect.y-150;
         if (y<0) y=0;      
         if (y+getViewport().getSize().height>=getViewport().getView().getHeight())
          y=getViewport().getView().getHeight()-getViewport().getSize().height;
         getViewport().setViewPosition(new Point(0,y));
      }
   }

   

   /**
    * Overwrites the add()-method in order to add new elements to the top level scroll pane only;
    * this is necessary because the contents of a CustomPanel dont have to be added to the panel
    * (which is a scroll pane) directly but to its child panel (which is a real JPanel)
    * @param arg0 component to add
    * @param arg1 layout information for that component
    */
   public void add(Component arg0, Object arg1)
   {
      if (mainPanel!=null) mainPanel.add(arg0,arg1);
      else super.add(arg0, arg1);
//      mainScroll.revalidate();
   }
   
   
  /**
   * Overwrites the add()-method in order to add new elements to the top level scroll pane only;
   * this is necessary because the contents of a CustomPanel dont have to be added to the panel
   * (which is a scroll pane) directly but to its child panel (which is a real JPanel)
   * @param arg0 component to add
   */
   public Component add(Component arg0)
   {
      Component c;
      
      c=mainPanel.add(arg0);
//      mainScroll.revalidate();
      return c;
   }

   
   
  /**
   * Overwrites the remove()-method in order to remove all elements from the top level scroll pane
   * only; this is necessary because the contents of a CustomPanel dont have to be added to the panel
   * (which is a scroll pane) directly but to its child panel (which is a real JPanel)
   * @param arg0 component to add
   */
   public void remove(Component arg0)
   {
      mainPanel.remove(arg0);
   }
   
   
   
/*   public void revalidate()
   {
      if (mainPanel!=null) mainPanel.revalidate();
      else super.revalidate();
   }*/
   
   
/*   public void invalidate()
   {
      if (mainPanel!=null) mainPanel.invalidate();
      else super.invalidate();
   }*/
   
   
/*   public void repaint()
   {
      if (mainPanel!=null) mainPanel.repaint();
      else super.repaint();
   }*/
   
   
   
   public void removeAll()
   {
      mainPanel.removeAll();
   }
   
  /**
   * This method is called either by the up/down commands of the remote control or by
   * pressing the cursor keys for up/down and modifies the selection of the elements
   * within the panel<BR>
   * This method has to be implemented by classes that inherit from CustomPanel
   * @param dir specifies the direction the selection has to be changed into, here
   *        values 1 and -1 are allowed 
   */
   public boolean switchHorizontal(int dir)
   {
      return false;      
   }
   
   
  /**
   * This method is called either by the left/right commands of the remote control or by
   * pressing the cursor keys for left/right. For these elements that support it this
   * additional selection can be used to alternate the contents or to display alternate
   * values<BR>
   * This method has to be implemented by classes that inherit from CustomPanel
   * @param dir specifies the direction the selection has to be changed into, here
   *        values 1 and -1 are allowed 
   */
   public boolean switchVertical(int dir)
   {
      return false;      
   }
   
   
   
  /**
   * This method is called either by other commands of the remote control or by
   * pressing other keyboard keys.<BR>
   * This method has to be implemented by classes that inherit from CustomPanel
   * @param dir specifies the direction the selection has to be changed into, here
   *        values 1 and -1 are allowed 
   */
   public void otherKeyEvent(int keyCode,char keyChar)
   {
   }
   
   
   
  /**
   * This method is called either numbers pressed on the remote control or on a
   * connected keyboard<BR>
   * This method has to be implemented by classes that inherit from CustomPanel
   * @param dir specifies the direction the selection has to be changed into, here
   *        values 1 and -1 are allowed 
   */
   public void sendNum(int num)
   {
   }
   
   
   
   /**
    * @deprecated use switchHorizontal() instead
    * @param idx
    * @return
    */
   public boolean setSelectedField(int idx)
   {
      return true;
   }

   

  /**
   * This method has to be implemented by classes that inherit from CustomPanel wherever it
   * is necessary: when a CustomPanel is removed by the main application first closePanel()
   * is called, then the panel is removed from the main window so that the standard view is
   * visible again and afterwards postClosePanel() is executed. Depending on what the
   * CustomPanel has displayed and depending on what is was used for possibly necessary
   * deinitializations have to be done here within closePanel() or within postClosePanel()
   */	
   public boolean closePanel()
   {
      return true;
   }

	
	
  /**
   * This method has to be implemented by classes that inherit from CustomPanel wherever it
   * is necessary: when a CustomPanel is removed by the main application first closePanel()
   * is called, then the panel is removed from the main window so that the standard view is
   * visible again and afterwards postClosePanel() is executed. Depending on what the
   * CustomPanel has displayed and depending on what is was used for possibly necessary
   * deinitializations have to be done here within postClosePanel() or within the preceding
   * closePanel()
   */		
   public void postClosePanel()
   {
   }
   
   
  /**
   * This method belons to the post/ClosePanel() methods: after the operation of a CustomPanel
   * is finished and after this paneol was removed from the main view using this method a
   * code is returned to the main application in order to inform about the operation and its
   * result
   * @return the result ID; here no predefinitions are made, the value and its meaning depend on
   *         the panel that inherits from CustomPanel
   */
   public int getResultID()
   {
      return -1;
   }
	
   
	
  /**
   * This method specifies if the panel can be aborted: if it returns true the
   * main application exits this panel in case the menu button was pressed on the
   * remote control or in case the F12 key was used  
   */
   public boolean canBeAborted()
   {
      return false;
   }
   
}
