/*
 * MenuField.java
 *
 * Copyright (c) 2008 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Fre
 * Software Foundation
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package base;

import javax.swing.*;

import java.awt.*;
import java.text.SimpleDateFormat;



/**
 * This class implements a single user interface element that can be selected by the user
 * with the cursor up/down keys. Such a MenuField can contain a single line of text for
 * functions, it can consist of combinations out of text lines and images for CD/DVD entries
 * and others. So a MenuField is always used when there are horizontal lines of data where
 * the user can choose from. 
 */
@SuppressWarnings("serial")
public class MenuField extends JFixedPanel 
{
   private String     text=null,dir;
   private int        id=-1,itemNum=-1;
   private JLabel     label1,label2=new JLabel(">"),label3=new JLabel(),label4=new JLabel();
   private TrackInfo  trackInfo=new TrackInfo(-1);
   private CDInfo     cdInfo=new CDInfo();
   private MediaInfo  mediaInfo=new DVDInfo();   
   private Color      customColor;
   private RecordInfo rInfo=null;
	
   

  /**
   * Constructor for a MenuField that displays information about a scheduled recording of the
   * internal VCR functionality
   * @param rInfo object that contains the scheduled recording information to be displayed to
   *        the user
   * @param id a unique identifier that can be used later to identify a selected MenuField
   * @param customColor color that has to be used for the labels when a MenuField is higlighted,
   *        when null is set the default color for highlighted texts is used
   */   
   public MenuField(RecordInfo rInfo,int id,Color customColor)
   {
      SimpleDateFormat   formatter =new SimpleDateFormat("dd.MM. HH:mm");
      this.id=id;
      this.rInfo=rInfo;
      this.customColor=customColor;
      
      setLayout(new GridBagLayout());
      label1=new JLabel(formatter.format(rInfo.stime*1000)+" "+rInfo.name);
      label2=new JLabel(rInfo.station);
      add(label1, new GridBagConstraints(0, 0, 1, 1, 0.9, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14, 1, 1), 0, 0));
      add(label2, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 14), 0, 0));
      setSelected(false);
   }
   
   
   
  /**
   * Constructor for a MenuField that display information about a CD
   * @param text the name of the album
   *        the user
   * @param id a unique identifier that can be used later to identify a selected MenuField
   * @param albumdir the storage position of that album
   * @param albumLevel currently unused, has to be removed
   * @param customColor color that has to be used for the labels when a MenuField is higlighted,
   *        when null is set the default color for highlighted texts is used
   */   
   public MenuField(String text,int id,String albumdir,boolean albumLevel,Color customColor)
   {
      JButton        button=new JButton();
      Image          img;
      Font           smallFont=new Font("",Font.PLAIN,22);
      
      this.text=text;
      this.id=id;
      this.customColor=customColor;
      if (this.customColor==null) this.customColor=PoddyFrame.stdentry;
            
//      setBackground(PoddyFrame.bgcolor);
      setLayout(new GridBagLayout());
      label1=new JLabel(text);
      add(label1, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
                                           ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14, 1, 1), 0, 0));
      img=Toolkit.getDefaultToolkit().createImage(albumdir+"/image_small.png");
      button.setIcon(new ImageIcon(img));
      button.setMargin(new Insets(0,0,0,0));
      button.setBorder(null);
      add(button,new GridBagConstraints(1, 0, 1, 3, 0.0, 0.0
                                           ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 2, 2, 2), 0, 0));
      add(label2, new GridBagConstraints(2, 0, 1, 3, 0.0, 0.0
                                           ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 14), 0, 0));
      label3.setFont(smallFont);
      add(label3, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
      label4.setFont(smallFont);         
      add(label4, new GridBagConstraints(0, 2, 1, 1, 1.0, 1.0
            ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, new Insets(1, 1, 1, 1), 0, 0));
      setSelected(false);
   }

   
   
  /**
   * Constructor for a MenuField that display information about the contents of a CD
   * @param text the name of the track
   *        the user
   * @param id a unique identifier that can be used later to identify a selected MenuField
   * @param trackdir the storage position of that track
   * @param customColor color that has to be used for the labels when a MenuField is higlighted,
   *        when null is set the default color for highlighted texts is used
   * @param hasSubEntries when this value is set to true a small arrow is displayed at the right
   *        side of the MenuField that informs the user about sub-entries that will open when this
   *        entry is selected (opening of these sub-entries has to be implemented by the main
   *        application)
   */   
   public MenuField(String text,int id,String trackdir,Color customColor,boolean hasSubEntries)
   {
      this.id=id;
      this.text=text;
      this.dir=trackdir;
      this.customColor=customColor;
      if (this.customColor==null) this.customColor=PoddyFrame.stdentry;
      
      setLayout(new GridBagLayout());
      label1=new JLabel(text);
      add(label1, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
                                           ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14, 1, 14), 0, 0));
      if (hasSubEntries) add(label2, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 14), 0, 0));
      setSelected(false);
   }
   
   
  /**
   * Constructor for a MenuField that displays general information; this constructor is used e.g.
   * for the top-level entries where the user can choose which part of the media center has to be
   * used
   * @param text the text for the menu
   *        the user
   * @param id a unique identifier that can be used later to identify a selected MenuField
   * @param customColor color that has to be used for the labels when a MenuField is higlighted,
   *        when null is set the default color for highlighted texts is used
   * @param hasSubEntries when this value is set to true a small arrow is displayed at the right
   *        side of the MenuField that informs the user about sub-entries that will open when this
   *        entry is selected (opening of these sub-entries has to be implemented by the main
   *        application)
   */   
   public MenuField(String text,int id,Color customColor,boolean hasSubEntries)
   {
      this.text=text;
      this.id=id;
      this.customColor=customColor;
      if (this.customColor==null) this.customColor=PoddyFrame.stdentry;

      setLayout(new GridBagLayout());
      label1=new JLabel(text);
      add(label1, new GridBagConstraints(0, 0, 1, 1, 0.9, 0.0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1,14, 1, 1), 0, 0));
      if (hasSubEntries) add(label2, new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(1, 1, 1, 14), 0, 0));
      setSelected(false);
   }
	
	

  /**
   * Retrieves the RecordInfo associated to a menu
   * @return the RecordInfo object associated to this menu item or null in case it is not a menu
   *         that displays RecordInfo data
   */	
   public RecordInfo getRecordInfo()
   {
      return rInfo;
   }
	
	
	
   public void setItemNum(int itemNum)
   {
      this.itemNum=itemNum;
   }

	
	
   public int getItemNum()
   {
      return itemNum;
   }

   
	
  /**
   * Retrieves the TrackInfo associated to a menu
   * @return the TrackInfo object associated to this menu item or null in case it is not a menu
   *         that displays a CDs TrackInfo data
   */	
   public TrackInfo getTrackInfo()
   {
      return trackInfo;
   }
	
	

  /**
   * @deprecated TODO: modify the CD-related constructors so that the CDInfo object can be set
   * there directly
   * This method can be used only for menu item types that display CD (album or track) data, it
   * can be used to set a full CDInfo structure to the menu
   * @param copy the CDInfo object that has to be associated with this menu item
   */
   public void setCDInfo(CDInfo copy)
   {
      cdInfo.set(copy);
      label3.setText("   "+cdInfo.genre+", "+cdInfo.year);
      label4.setText("");
   }
	
	
	
  /**
   * Retrieves the CDInfo associated to a menu
   * @return the CDInfo object associated to this menu item or null in case it is not a menu
   *         that displays CDInfo data
   */	
   public CDInfo getCDInfo()
   {
      return cdInfo;
   }

   
   
  /**
   * @deprecated TODO: modify the CD-related constructors so that the TrackInfo object can be set
   * there directly
   * This method can be used only for menu item types that display CD (album or track) data, it
   * can be used to set a full TrackInfo structure to the menu
   * @param copy the TrackInfo object that has to be associated with this menu item
   */
   public void setTrackInfo(TrackInfo copy)
   {
      if (copy==null) trackInfo=null;
      else trackInfo.set(copy);
   }
	
	
	
  /**
   * Retrieves the MediaInfo associated to a menu
   * @return the MediaInfo object associated to this menu item or null in case it is not a menu
   *         that displays MediaInfo-related data
   */	
   public MediaInfo getMediaInfo()
   {
      return mediaInfo;
   }
	
	
	
  /**
   * @deprecated TODO: modify the Media-related constructors so that the MediaInfo object can be set
   * there directly
   * This method can be used only for menu item types that display media data, it
   * can be used to set a full MediaInfo structure to the menu
   * @param copy the MediaInfo object that has to be associated with this menu item
   */
   public void setMediaInfo(MediaInfo copy)
   {
      if (copy instanceof DVDInfo)
      {
         ((DVDInfo)mediaInfo).set((DVDInfo)copy);
         label3.setText("   "+((DVDInfo)mediaInfo).genre+", "+((DVDInfo)mediaInfo).year);
         label4.setText("   "+((DVDInfo)mediaInfo).time+" min");
      }
      else mediaInfo.set(copy);
   }
	
	

  /**
   * Sets the full path to the directory where data related to the menu item are stored into
   * @param dir the filesystem path to the storage location
   */	
   public void setDir(String dir)
   {
      this.dir=dir;
   }
	
	

  /**
   * Retrieves the path in filesystem to data related to this menu item
   * @return the path to the storage directory of the data or null in case this is not
   *         applicable for this menu item type
   */	
   public String getDir()
   {
      return dir;
   }
	
   
   
  /**
   * Returns the unique identifier assigned to this menu item
   * @return the id set during consturction of the object
   */
   public int getID()
   {
      return id;
   }
	
	

  /**
   * Retrieves the text associated with a menu item. This method is called by the main application
   * whenever an entry is selected and higlighted. In this case the value returned here is sent to
   * an optionally connected LCD so that the application could be used also without a TV connected.
   * @return the text of this menu item
   */	
   public String getText()
   {
      return text;
   }

   

  /**
   * Sets the selection and highlighting-state of this menu field.
   * @param selected true when this item is selected and has to be highlighted,
   *        false otherwise
   */
   public void setSelected(boolean selected)
   {
      super.setSelected(selected);
      if (selected)
      {
//		  setBackground(PoddyFrame.highlight);
         label1.setForeground(Color.WHITE);
         label2.setForeground(Color.WHITE);
         if (label3!=null) label3.setForeground(Color.lightGray.brighter());
         if (label4!=null) label4.setForeground(Color.lightGray.brighter());
         PoddyFrame.MyWin.setLCDText(getText(),null);
      }
      else
      {
//	  setBackground(PoddyFrame.bgcolor);
         label1.setForeground(customColor);
         label2.setForeground(customColor);
         if (label3!=null) label3.setForeground(PoddyFrame.stdentry);
         if (label4!=null) label4.setForeground(PoddyFrame.stdentry);
      }
   }
	
	
  /**
   * @deprecated duplicate of getText()
   */	
   public String getTitle()
   {
      return text;	
   }
	
}
