/*
 *	Screen.java 1.1 03.01.2009
 *
 * Copyright (c) 2004 Gavin & Georgina Sallery
 * Copyright (c) 2008-2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 */

package com.vwp.jLCD;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * The screen is the base for all further elements. When a screen was added to an LCD object some widgets
 * can be added to that screen. These widgets then contain the displayed data and influence what is visible
 * on the display 
 */
public class Screen {
	public static final int PRIORITY_HIDDEN=0;       /** priority value for a hidden screen, this one will never be visible */
	public static final int PRIORITY_FOREGROUND=1;   /** lowest priority value for a foreground screen, this specifies an active screen, only the one with the highest foreground priority value will be visible */
	public static final int PRIORITY_INFO=65;        /** lowest priority value for a info screen, if no foreground screens are available but more than one info screen they will rotate */
	public static final int PRIORITY_BACKGROUND=193; /** lowest priority value for a background screen, this screen is only visible when no normal foreground screens exist; if there are more than one background screen they will rotate */

	private String    name="";            //name of screen
	private int       width;              //width of screen
	private int       height;             //height of screen
	private int       duration;           //Duration the screen is displayed
	private int       priority=3;
	private Integer   id = null;      // Unique identifier for this screen
	private boolean   heartbeat;       //screen mode
	private LCD       lcd = null;
	private int       widgetCount = 0;
	private ArrayList listeners = null;

	/**
	 * Default constructor for a new screen
	 */
    public Screen() 
    {
       listeners = new ArrayList();
    }

    
    
    /**
     * Constructor for a new screen with a default priority of 3
     * @param name the name for the screen
     */
    public Screen(String name) 
    {
       this.name = name;
       listeners = new ArrayList();
       update();
    }

    
    
    /**
     * The priority of a screen decides if and on what level such a screen is displayed
     * @return the current priority of that screen
     */
	public int getPriority() {
		return priority;
	}

	
	
    /**
     * The priority of a screen decides if and on what level such a screen is displayed
     * @param priority the new priority for that screen
     */
	public void setPriority(int priority) 
	{
		this.priority = priority;
		update();
	}

	
	
	/**
	 * The heartbeat feature is some kind of alive-information that shows that the system
	 * is still up and running
	 * @return true if heartbeat is turned on, false otherwise
	 */
	public boolean getHeartbeat() {
		return heartbeat;
	}

	
	
	/**
	 * The heartbeat feature is some kind of alive-information that shows that the system
	 * is still up and running
	 * @param heartbeat if this parameter is set to true the heartbeat feature will be turned on
	 */
	public void setHeartbeat(boolean heartbeat) {
		this.heartbeat = heartbeat;
		update();
	}

	
	
	/**
	 * @return the name of the screen
	 */
	public String getName() {
		return name;
	}

	
	
	/**
	 * @param name the new name for that screen
	 */
	public void setName(String name) {
		this.name = name;
		update();
	}

	
	
	/**
	 * A screen has a size that can be independent from the displays size
	 * @return the current width of the screen
	 */
	public int getWidth() {
		return width;
	}

	
	
	/**
	 * A screen has a size that can be independent from the displays size
	 * @param width the new width this screen needs to have
	 */
	public void setWidth(int width) {
		this.width = width;
		update();
	}

	
	
	/**
	 * A screen has a size that can be independent from the displays size
	 * @return the current height of the screen
	 */
	public int getHeight() {
		return height;
	}

	
	
	/**
	 * A screen has a size that can be independent from the displays size
	 * @param height the new width this screen needs to have
	 */
	public void setHeight(int height) {
		this.height = height;
		update();
	}

	
	
	public int getDuration() {
		return duration;
	}

	
	
	public void setDuration(int duration) {
		this.duration = duration;
		update();
	}

	
	
	public Integer getID() {
		return id;
	}

	
	
	public void addListener(ScreenListener listener) {
		listeners.add(listener);
	}

	
	
	public void removeListener(ScreenListener listener) {
		listeners.remove(listener);
	}

	
	
	public void listen() {
		// The server is listening to us. Might want to call our listeners.
		Iterator i = listeners.iterator();
		while (i.hasNext())
			((ScreenListener)i.next()).listen();
	}

	
	
	public void ignore() {
		// The server is ignoring us. Might want to call our listeners.
		Iterator i = listeners.iterator();
		while (i.hasNext())
			((ScreenListener)i.next()).ignore();
	}

	
	
	/**
	 * Only after a widget is added to a screen the display will show something. Using this
	 * function such a widget - that contains the data to be displayed - can be added
	 * @param w the widget to be added to the screen
	 */
	public void addWidget(Widget w) {
		if (lcd == null)
			throw new IllegalStateException("Impossible to add widgets unless screen has been added to LCD");
		lcd.write("widget_add " + id + " " + widgetCount + " " + w.getType());
		w.added(lcd, this, widgetCount);
		widgetCount++;
	}

	
	
	void added(LCD lcd, int screenID) {
		this.id = new Integer(screenID);
		this.lcd = lcd;
		update();
	}

	
	
	private void update() {
		if (lcd != null)
		{
			String s;
			
			if (heartbeat) s="on";
			else s="off";
			lcd.write("screen_set " + id + " -priority " + priority + " -name \"" + name +
			          "\" -duration " + duration + " -wid " + width + " -hgt " + height + " -heartbeat " + s);
		}
	}
}
