package com.vwp.swing;

import javax.swing.*;
import javax.swing.text.*;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.awt.event.*;

/*
 *	XIntegerField.java 1.3 18.05.2009
 *
 * Copyright (c) 2002-2004 Virtual Worlds Productions
 * Email: virtual_worlds@gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds@gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

/*
Reviewed:
11. 06. 2003
*/



@SuppressWarnings("serial")
/*******************************************************************************
 * This class handles input fields which can be used to enter doubles only. All
 * other characters will be rejected by this class.
 */
public class XIntegerField extends JTextField
   {
   private NumberFormat integerFormatter;
   int MinVal=0,MaxVal=0x7FFFFFFF;



  /*****************************************************************************
   * Constructs a new XIntegerField with an initial value of 0
   */
   public XIntegerField()
      {
      super(4);
      integerFormatter = NumberFormat.getNumberInstance(Locale.US);
      integerFormatter.setParseIntegerOnly(true);
      setValue(0);
      enableEvents(java.awt.AWTEvent.FOCUS_EVENT_MASK);
      }



  /*****************************************************************************
   * Constructs a new XIntegerField
   * @param value The initial value
   * @param columns The width of the field
   */
   public XIntegerField(int value, int columns)
      {
      super(columns);
      integerFormatter = NumberFormat.getNumberInstance(Locale.US);
      integerFormatter.setParseIntegerOnly(true);
      setValue(value);
      }



  /*****************************************************************************
   * Constructs a new XIntegerField
   * @param value The initial value
   * @param MinVal The minimum allowed value which can be entered; smaller
   *        values will automatically be set to MinVal
   * @param MaxVal The maximum allowed value which can be entered; greater
   *        values will automatically be set to MaxVal
   */
   public XIntegerField(int value, int MinVal, int MaxVal)
      {
      super(4);
      set(value,MinVal,MaxVal);
      }



  /*****************************************************************************
   * Constructs a new XIntegerField
   * @param value The initial value
   * @param columns The width of the field
   * @param MinVal The minimum allowed value which can be entered; smaller
   *        values will automatically be set to MinVal
   * @param MaxVal The maximum allowed value which can be entered; greater
   *        values will automatically be set to MaxVal
   */
   public XIntegerField(int value, int columns, int MinVal, int MaxVal)
      {
      super(columns);
      set(value,MinVal,MaxVal);
      }



   private void set(int value, int MinVal, int MaxVal)
      {
      integerFormatter = NumberFormat.getNumberInstance(Locale.US);
      integerFormatter.setParseIntegerOnly(true);
      this.MinVal=MinVal;
      this.MaxVal=MaxVal;
      if (value<MinVal) value=MinVal;
      if (value>MaxVal) value=MaxVal;
      setValue(value);
      }



  /*****************************************************************************
   * Sets a new maximum value
   * @param MaxVal The maximum allowed value which can be entered; greater
   *        values will automatically be set to MaxVal
   */
   public void setMaxValue(int MaxVal)
      {
      this.MaxVal=MaxVal;
      if (getValue()>MaxVal) setValue(MaxVal);
      }



  /*****************************************************************************
   * Sets a nem minimum value
   * @param MinVal The minimum allowed value which can be entered; smaller
   *        values will automatically be set to MinVal
   */
   public void setMinValue(int MinVal)
      {
      this.MinVal=MinVal;
      if (getValue()<MinVal) setValue(MinVal);
      }



  /*****************************************************************************
   * Returns the actual maximum value
   */
   public int getMaxValue()
      {
      return this.MaxVal;
      }



  /*****************************************************************************
   * Returns the actual minimum value
   */
   public int getMinValue()
      {
      return this.MinVal;
      }



   public String getText()
      {
      throw new RuntimeException("getText() not supported by XIntegerField class!");
      }



  /*****************************************************************************
   * Returns the actual value
   */
   public int getValue()
      {
      int retVal = 0;
      try
         {
         retVal = integerFormatter.parse(super.getText()).intValue();
         }
      catch (ParseException e) { }
      return retVal;
      }



  /*****************************************************************************
   * Sets the actual maximum value
   * @param value The new actual value which has to be displayed
   */
   public void setValue(int value)
      {
      setText(integerFormatter.format(value));
      }



  /*****************************************************************************
   * Clears the IntegerField so that no number can be seen. This sets a state
   * which is equal to value of 0 but in which this 0 isn't visible; this method
   * may be used in some special cases only
   */
   public void clearValue()
      {
      setText("");
      }



   protected Document createDefaultModel()
      {
      return new XIntegerDocument();
      }



   public void processFocusEvent(FocusEvent e)
      {
      int ActVal;

      try
         {
         ActVal=getValue();
         if (ActVal>MaxVal) setText(String.valueOf(MaxVal));
         else if (ActVal<MinVal)  setText(String.valueOf(MinVal));
         super.processFocusEvent(e);
         }
      catch (NullPointerException npe) {} // catches NPEs caused by a focusLost-timing Problem
      }



   protected class XIntegerDocument extends PlainDocument
      {
      int ActVal;

      public void insertString(int offs,String str,AttributeSet a)
      throws BadLocationException
         {
         char[] source = str.toCharArray();
         char[] result = new char[source.length];
         int j = 0;

         for (int i = 0; i < result.length; i++)
            {
            if ((Character.isDigit(source[i])) || (source[i]=='-')) result[j++] = source[i];
            }
         super.insertString(offs,new String(result, 0, j), a);
         }

      }

   }

