/*
 *	XProgressBar.java 1.00 21.01.2009
 *
 * Copyright (c) 2009 Virtual Worlds Productions
 * Email: virtual_worlds(at)gmx.de
 *
 * This program is free software; you can redistribute it and/or modify  it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * Please contact virtual_worlds(at)gmx.de if you are interested in a source
 * package which is not licensed under the GPL
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. VIRTUAL WORLDS PRODUCTIONS AND ITS
 * LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES.
 * IN NO EVENT WILL VIRTUAL WORLDS PRODUCTIONS OR ITS LICENSORS BE LIABLE FOR
 * ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
 * CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS
 * OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
 * SOFTWARE, EVEN IF VIRTUAL WORLDS PRODUCTIONS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

package com.vwp.swing;

import java.awt.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * This is a simple progress bar that provides - comparing to the JProgresssBar - a more nice
 * layout and a very smooth scrolling of the bar.
 */
public class XProgressBar extends JComponent implements Runnable 
{
	public static final int MODE_STANDARD=1; /** standard operation mode, the progress bar is updated every time a new value is set */
	public static final int MODE_TIME=2;     /** time operation mode, the values are interpreted as milliseconds and the progress bar moves automatically depending on the elapsed time */

	private long   min,max,value,startTime=-1;
	private Thread myThread;

	/**
	 * The default constructor creates a XProgressBar object in standard mode with a minimum value of 0, a maximum
	 * value of 100 and a state value of 0
	 */
	public XProgressBar()
	{
		super();
		init(0,100,MODE_STANDARD);
	}
	
	/**
	 * This creates a new XProgressBar with the given minimum and maximum values. The current state
	 * value is set to the same value like min.
	 * @param min the minimum value the bar should visualize
	 * @param max the maximum possible value
	 */
	public XProgressBar(long min,long max,int mode)
	{
		super();
		init(min,max,mode);
	}
	
	
	
	private void init(long min, long max,int mode)
	{
	   setBorder(new BevelBorder(BevelBorder.LOWERED));
	   this.min=min;
	   this.value=1000;//min;
	   this.max=max;
	   setForeground(Color.BLUE);
	   if (mode==MODE_TIME)
	   {
		   myThread=new Thread(this);
		   myThread.start();
	   }
	}
	
	
	
	public void restart()
	{
	   startTime=System.currentTimeMillis();
	}
	
	
	
	public void stop()
	{
	   startTime=-1;
	}
	
	
	
	public void setMinimum(long min)
	{
	   this.min=min;
	}

	
	
   public void setMaximum(long max)
   {
      this.max=max;
   }
   
   

	public void run()
	{
      long currTime;
		
		while (true)
		{		   
		   if ((value<max) && (startTime>0))
		   {
    		  currTime=System.currentTimeMillis();
	    	  value+=(currTime-startTime);
	    	  startTime=currTime;
	        repaint();
	        if (value>=max) startTime=-1;
		   }
		   com.vwp.util.Tools.threadSleep(150);
		}
	}
	
	/**
	 * This method can be used to set a new value and to redraw the bar with that value;
	 * in time mode the value is re-set to the specified time, so this function can be used
	 * for a re-synchronization of the elapsed amd displayed time
	 * @param value the new value that has to be visualized by the bar
	 */
	public void setValue(long value)
	{
		if (value>max) value=max; 
		this.value=value;
	}

	
	
	public void paintComponent(Graphics g)
	{
		int w=getSize().width-6,h=getSize().height-6;
		int fillw;
		if ((w>0) && (h>0))
		{
		   if (value<min) fillw=0;
		   else fillw=(int)(((w*1.0)/(max-min))*(value-min));
		   g.setColor(getForeground());
		   g.fillRect(3,3,fillw,h);
		   if (fillw<w)
		   {
		      g.setColor(getBackground());
		      g.fillRect(3+fillw,3,w-fillw,h);
		   }
		}
		paintBorder(g);
	}
}
